''' CLI Parser

Module of tarpy.
Usage:
    tarpy [ROOT] [TARGET] -m {'w', 'e'} [--exclude_file] [--compression]

'''

import argparse
from typing import Any
from tarpy.tarhandler import TarArchive


def cliparser() -> Any:
    ''' CLI Parser

    Function takes parameters like defined below and
    returns the args.
    '''
    parser = argparse.ArgumentParser(
            description = 'Different TAR Archive operations.',
            epilog = 'Python3 version.',
            )
    parser.add_argument(
            'ROOT',
            type=str,
            help='The (starting) directory to archive respectively the file to extract.'
            )
    parser.add_argument(
            'TARGET',
            type=str,
            help='The path where archive will be written to.'
            )
    parser.add_argument(
            '-m',
            choices=['w', 'e'],
            help = 'Which Operation to run.'
            )

    parser.add_argument(
            '-ex-f',
            '--exclude-file',
            metavar='PATH',
            help='File with exclusions defined.'
            )

    parser.add_argument(
            '-c',
            '--compression',
            default='gz',
            choices=['gz', 'bz', 'lz'],
            help='The compression method.'
            )

    return parser.parse_args()

def cli() -> Any:
    ''' Core function of CLI
    '''
    args = cliparser()

    root = args.ROOT
    target = args.TARGET
    mode = args.m

    # Default Variables.
    ex_file = args.exclude_file
    compression = args.compression

    handler = TarArchive(root, target, ex_file, compression)
    print(handler)
    return handler(mode)


if __name__ == '__main__':

    # cli()
    print(cli.__annotations__)



''' Config

Config Variables for tarpy Package.
'''


import os
from datetime import datetime
from pathlib import Path


# Define Static Variables
SYSTEM = os.uname().sysname
SYSTEMNAME = os.uname().release
MACHINE = os.uname().machine
HOST = os.uname().nodename
ARCHITECTURE = os.uname().machine
OWNDIR = os.path.dirname(os.path.realpath(__file__))
TODAY = datetime.today().strftime('%Y-%m-%d')
EXCLUDE_FILE = '/usr/local/include/backup_exclusions.txt'

# The Config File
CONFIG_FILE = Path(Path.home() / '.tarpy/user.conf').resolve()


# Check for Config
if not os.path.exists(os.fspath(CONFIG_FILE)):
    # conf_dialog()
    ...

# Read Config and Set Variables
...



#!/usr/bin/python3
# -*- coding: utf-8 -*-

''' Make TAR Archive

'''

import tarfile

# Internal Imports
from tarpy import config
from tarpy.utils import real_path
from tarpy.filters import filtered_walk


class TarArchive:
    ''' TAR Archive

    Attributes
    ----------
    EXCLUSIONS : list
        A list of strings for extra exclusions.

    Parameters
    ----------
    start_point : str
        The most top path to start archiving from.
        For extracting TAR archives it is the file-
        name of the Archive to esxtract.
    target_dir : str
        The target Path of the TAR Operation.
    exclude_file : str
        The Path of the file with defined exclusions.
    compression : str
        Compression Mode.
    '''
    EXCLUSIONS = []

    def __init__(self,
            start_point: str,
            target_dir: str,
            exclude_file: str,
            compression: str,
            ) -> None:
        self.start_point = str(real_path(start_point))
        self.target_dir = str(real_path(target_dir))
        self.exclude_file = str(real_path(exclude_file))
        self.compression = compression

        # Make sure Exclusions are set.
        self.add_exclusions()

    def add_exclusions(self, *excludes) -> None:
        ''' Set own Exclusions

        There have to be some program related
        exclusions (e.g. the directory of this
        script) independently from the user
        defined exclusions from the file.

        Parameters
        ----------
        *excludes : tuple
            A Tuple containing exclusions re-
            presented by strings.
        '''
        if excludes:
            for elem in excludes:
                self.EXCLUSIONS.append(elem)

        # self.EXCLUSIONS.append(str(self.target_dir))

        # If there is a Blacklist file:
        if self.exclude_file:
            with open(self.exclude_file, 'r') as ex_f:
                for exclude in ex_f.readlines():
                    if exclude != '\n':
                        self.EXCLUSIONS.append(exclude.strip('\n'))

    def tar_writer(self) -> None:
        ''' Write TAR Archive

        Method for writing the archive.
        '''
        name = real_path(
                f'{self.target_dir}/{config.SYSTEMNAME}_{config.TODAY}.tar.{self.compression}'
                )
        with tarfile.open(name, f'w:{self.compression}') as tar_f:
            for path in filtered_walk(self.start_point, self.EXCLUSIONS):
                # The recursive option
                # should be set to False.
                tar_f.add(path, recursive=False)

    def tar_extractor(self) -> None:
        ''' Extract TAR Archive

        Method for extracting the archive.
        '''
        ...

    def __repr__(self) -> str:
        return (f'Class: {self.__class__.__name__!r}\n'
                f'Start from: {str(self.start_point)!r}\n'
                f'Target Directory: {str(self.target_dir)!r}\n'
                f'Compression: {self.compression!r}\n'
                f'Exclude File: {str(self.exclude_file)!r}\n'
                )

    def __str__(self) -> str:
        return (f'Source: {str(self.start_point)}\n'
                f'Target: {str(self.target_dir)}\n'
                f'Compression: {self.compression}\n'
                f'Exclusions: {self.EXCLUSIONS}\n'
                )

    def __call__(self, mode) -> str:
        ''' Make this class callable
        '''

        if mode == 'w':
            self.tar_writer()
        elif mode == 'r':
            self.tar_extractor()



''' Utils

Module of tarpy.
'''

import os
import pathlib


def real_path(path: str) -> pathlib.PosixPath[str]:
    ''' Real Paths
    '''
    try:
        return pathlib.Path(path).resolve()
    except TypeError:
        pass

def check_existence(to_check: str) -> bool:
    ''' Check if File / Directory
        exists or not
    '''
    if not os.path.exists(to_check):
        return False
    return True

def check_symlink(to_check: str) -> bool:
    ''' Check if File is a Symlink
    '''
    return pathlib.Path(to_check).is_symlink()



''' setup.py
'''


import setuptools


setuptools.setup(
        name='tarpy',
        version='0.0.1',
        author='m1ghtfr3e',
        description='Python Version of tar.',
        packages=setuptools.find_packages(),
	entry_points={
            'console_scripts' : [
                'tarpy = tarpy.cli.run:cli'
                ]
        },

        classifiers=[
            'Programming Language :: Python :: 3',
            'Environment :: Console',
        ],

        python_requires='>=3',
        project_urls={
            'Source' : 'https://github.com/m1ghtfr3e/tarpy'
            }
)



''' Filter Paths Out

Related functions to filter out
paths.
'''

import fnmatch
import os
from collections.abc import Callable, Iterable
from pathlib import Path
from tarpy.utils import check_symlink


def should_exclude(path: str, exclusions: list[str]) -> bool:
    ''' Should Exclude

    This module checks against fnmatch and
    returns a bool.

    Parameters
    ----------
    path : str
        A Path to check against fnmatch.
    exclusions : list[str]
        The list with exclusions.

    Returns
    -------
     : bool
        True if it matches at least on of
        the exclusions.
        False if not.
    '''
    return any(fnmatch.fnmatch(path, exclude) for exclude in exclusions)

def filter_paths(source: str, rule: Callable[str], exclusions: list[str]) -> Iterable[str]:
    ''' Filter Paths

    It is filtering out paths like defined in
    self.should_exclude.
    Is returning then the path as a string.

    Parameters
    ----------
    source : str
        The corresponding Path.
    rule : Callable[str]
        A function which is checking if to
        filter or not.
    exclusions : list[str]
        The list of exclusions.

    Returns
    -------
        : Iterable[str]
        A Generator with only "good" Paths.
    '''
    if (
            not rule(source, exclusions) and
            source is not None and
            not check_symlink(source)
        ):
        yield source

def filtered_walk(source: Path, exclusions: list[str]) -> Iterable[str]:
    ''' Filtered Walk over Paths

    This method acts like a generator.
    It yields all paths except those filtered out liked defined
    in the EXCLUSIONS.

    Returns
    -------
        : Iterable[str]
        A Generator giving the filtered Paths.
    '''
    for root, dirs, files in os.walk(source):
        try:
            for directory in dirs:
                tmp_dir = os.path.realpath(os.path.join(root, directory))
                yield from filter_paths(tmp_dir, should_exclude, exclusions)
            for elem in files:
                tmp_file = os.path.realpath(os.path.join(root, elem))
                yield from filter_paths(tmp_file, should_exclude, exclusions)

        except PermissionError:
            continue
        except FileNotFoundError:
            continue
